package com.ruoyi.common.utils.sm4;
import org.apache.commons.codec.binary.Base64;
import java.nio.charset.StandardCharsets;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 国密SM4工具类（自行实现国密SM4算法）
 * <p>有些项目中，需要单独实现国密SM4算法，
 * <p>比如：已经集成了老版本的bouncycastle，但老版本的bouncycastle无法实现国密SM4算法。但你又不能升级它，而你又需要国密SM4算法。
 *
 * <p>参考：</p>
 * <ol>
 *     <li><a href="https://blog.csdn.net/sunxmwebstudy/article/details/114699704">SM4对称加密算法及Java实现</a></li>
 *     <li><a href="https://www.jianshu.com/p/5ec8464b0a1b">SM4对称加密算法</a></li>
 * </ol>
 *
 * <p>另外有开源项目实现了国密的全部算法：</p>
 * <a href="https://github.com/GmSSL/GmSSL-Java">GmSSL-Java(基于JNI)</a>
 * <a href="https://gitee.com/openeuler/bgmprovider">BGMProvider</a>
 * <a href="https://github.com/Tencent/TencentKonaSMSuite">腾讯Kona国密套件</a>
 */
public class SM4Utils {

    public static byte[] encryptData_ECB(String plainText, byte[] keyBytes) throws Exception {
        SM4_Context ctx = new SM4_Context();
        ctx.isPadding = true;
        ctx.mode = SM4_Body.SM4_ENCRYPT;

        SM4_Body sm4Body = new SM4_Body();
        sm4Body.sm4_setkey_enc(ctx, keyBytes);
        return sm4Body.sm4_crypt_ecb(ctx, plainText.getBytes(StandardCharsets.UTF_8));
    }

    public static String decryptData_ECB(String cipherText, byte[] keyBytes) throws Exception {
        if (cipherText != null && !cipherText.trim().isEmpty()) {
            Pattern p = Pattern.compile("\\s*|\t|\r|\n");
            Matcher m = p.matcher(cipherText);
            cipherText = m.replaceAll("");
        }

        return decryptData_ECB(Base64.decodeBase64(cipherText),keyBytes);
    }

    public static String decryptData_ECB(byte[] cipherByte, byte[] keyBytes) throws Exception {
        SM4_Context ctx = new SM4_Context();
        ctx.isPadding = true;
        ctx.mode = SM4_Body.SM4_DECRYPT;

        SM4_Body sm4Body = new SM4_Body();
        sm4Body.sm4_setkey_dec(ctx, keyBytes);
        byte[] decrypted = sm4Body.sm4_crypt_ecb(ctx, cipherByte);
        return new String(decrypted, StandardCharsets.UTF_8);
    }

    public static byte[] encryptData_CBC(String plainText, byte[] keyBytes, byte[] ivBytes) throws Exception {
        SM4_Context ctx = new SM4_Context();
        ctx.isPadding = true;
        ctx.mode = SM4_Body.SM4_ENCRYPT;

        SM4_Body sm4Body = new SM4_Body();
        sm4Body.sm4_setkey_enc(ctx, keyBytes);

        return sm4Body.sm4_crypt_cbc(ctx, ivBytes, plainText.getBytes(StandardCharsets.UTF_8));
    }

    public static String decryptData_CBC(String cipherText, byte[] keyBytes, byte[] ivBytes) throws Exception {
        if (cipherText != null && !cipherText.trim().isEmpty()) {
            Pattern p = Pattern.compile("\\s*|\t|\r|\n");
            Matcher m = p.matcher(cipherText);
            cipherText = m.replaceAll("");
        }

        return decryptData_CBC(Base64.decodeBase64(cipherText),keyBytes,ivBytes);
    }

    public static String decryptData_CBC(byte[] cipherByte, byte[] keyBytes, byte[] ivBytes) throws Exception {
        SM4_Context ctx = new SM4_Context();
        ctx.isPadding = true;
        ctx.mode = SM4_Body.SM4_DECRYPT;

        SM4_Body sm4Body = new SM4_Body();
        sm4Body.sm4_setkey_dec(ctx, keyBytes);

        byte[] decrypted = sm4Body.sm4_crypt_cbc(ctx, ivBytes, cipherByte);

        return new String(decrypted, StandardCharsets.UTF_8);
    }

    public static void main(String[] args) throws Exception {
        String plainText = "abc";
        // key 16 位字符串
        String key = "UVMxWFBaZXVSNm9YVTU0dg==";
        byte[] keyByte = Base64.decodeBase64(key);

        System.out.println("ECB模式加密");
        byte[] cipherTextByte = SM4Utils.encryptData_ECB(plainText, keyByte);
        String cipherText = Base64.encodeBase64String(cipherTextByte);
        System.out.println("密文: " + cipherText);//qtu7LHVpilXxaz8mPW1VjQ==
        String plainText2 = SM4Utils.decryptData_ECB(cipherText, keyByte);
        System.out.println("解密: " + plainText2);

        System.out.println("CBC模式加密");
        String iv = "3131313131313131";
        //byte[] ivByte = Base64.decodeBase64(iv);
        byte[] ivByte = iv.getBytes();

        byte[] cipherText2Byte = SM4Utils.encryptData_CBC(plainText,keyByte,ivByte);
        String cipherText2 = Base64.encodeBase64String(cipherText2Byte);
        System.out.println("加密密文: " + cipherText2);
        String plainText3 = SM4Utils.decryptData_CBC(cipherText2Byte,keyByte,ivByte);
        System.out.println("解密明文: " + plainText3);
    }
}